/*
  This file is under public domain.
*/

#include <windows.h>
#include <stdlib.h>
#include <string.h>
#define KMELEON_PLUGIN_EXPORTS
#include <kmeleon_plugin.h>

#if (KMEL_PLUGIN_VER) > 0x0201 /* >= 1.0 */
#include <nsIWebBrowser.h>
#include <nsIWebBrowserChrome.h>
#include <nsIEmbeddingSiteWindow.h>
#include "miniCOMPtr.h"
#endif

#include "utils_io.h"
#include "utils_os.h"
#include "utils_dialog.h"
#include "utils_reg.h"
#include "utils_string.h"
#include "utils_perm.h"
#include "utils_xpath.h"
#include "utils_sendkey.h"

#define PLUGIN_NAME "Macro Utilities Plugin"

long DoMessage(const char *to, const char *from, const char *subject,
               long data1, long data2);

kmeleonPlugin kPlugin = {
	KMEL_PLUGIN_VER,
	PLUGIN_NAME,
	DoMessage
};

typedef void (*PLUGINMSG)(char *, char *);
typedef long (*PLUGINMSGEX)(char *);

typedef enum {TYPE_MSG, TYPE_MSGEX} API_TYPE;

typedef struct {
	const char *signature;
	API_TYPE type;
	union {
		PLUGINMSG pluginmsg;
		PLUGINMSGEX pluginmsgex;
	};
} PLUGINAPI;


HINSTANCE get_browser_instance()
{
	return kPlugin.hParentInstance;
}


HINSTANCE get_plugin_instance()
{
	return kPlugin.hDllInstance;
}


HWND get_browser_hwnd()
{
	HWND hwnd = GetActiveWindow();
#if (KMEL_PLUGIN_VER) > 0x0201 /* >= 1.0 */
	nsresult rv;
	
	miniCOMPtr<nsIWebBrowser> browser;
	if (! kPlugin.kFuncs->GetMozillaWebBrowser(NULL, &browser)) {
		return hwnd;
	}
	
	miniCOMPtr<nsIWebBrowserChrome> chrome;
	rv = browser->GetContainerWindow(&chrome);
	NS_ENSURE_SUCCESS(rv, hwnd);
	NS_ENSURE_TRUE(chrome, hwnd);
	
	miniCOMPtr<nsIEmbeddingSiteWindow> esw;
	rv = chrome.QueryInterface(esw);
	NS_ENSURE_SUCCESS(rv, hwnd);
	NS_ENSURE_TRUE(esw, hwnd);
	
	esw->GetSiteWindow((void **) &hwnd);
#endif
	return hwnd;
}


const char *translate(const char *text)
{
#if (KMEL_PLUGIN_VER) > 0x0201 /* >= 1.0 */
	return kPlugin.kFuncs->Translate(text);
#else
	return text;
#endif
}


char *utils_EncodeUTF8(char *data)
{
	return kPlugin.kFuncs->EncodeUTF8((char *)data);
}

char *utils_DecodeUTF8(char *data)
{
	return kPlugin.kFuncs->DecodeUTF8((char *)data);
}


static PLUGINAPI funcs[] = {
	{"EncodeUTF8", TYPE_MSGEX, (PLUGINMSG) utils_EncodeUTF8},
	{"DecodeUTF8", TYPE_MSGEX, (PLUGINMSG) utils_DecodeUTF8},
	{"exist", TYPE_MSGEX, (PLUGINMSG) utils_exist},
	{"copyfile", TYPE_MSG, (PLUGINMSG) utils_copyfile},
	{"rename", TYPE_MSG, (PLUGINMSG) utils_rename},
	{"unlink", TYPE_MSG, (PLUGINMSG) utils_unlink},
	{"read", TYPE_MSGEX, (PLUGINMSG) utils_read},
	{"write", TYPE_MSG, (PLUGINMSG) utils_write},
	{"append", TYPE_MSG, (PLUGINMSG) utils_append},
	{"mkdir", TYPE_MSGEX, (PLUGINMSG) utils_mkdir},
	{"getenv", TYPE_MSGEX, (PLUGINMSG) utils_getenv},
	{"setenv", TYPE_MSG, (PLUGINMSG) utils_setenv},
	{"exec", TYPE_MSG, (PLUGINMSG) utils_exec},
	{"execex", TYPE_MSGEX, (PLUGINMSG) utils_execex},
	{"sleep", TYPE_MSG, (PLUGINMSG) utils_sleep},
	{"strftime", TYPE_MSGEX, (PLUGINMSG) utils_strftime},
	{"ActivateWindow", TYPE_MSG, (PLUGINMSG) utils_activate_window},
	{"GetOSVer", TYPE_MSGEX, (PLUGINMSG) utils_getosver},
	{"GetOpenFileName", TYPE_MSGEX, (PLUGINMSG) utils_GetOpenFileName},
	{"GetSaveFileName", TYPE_MSGEX, (PLUGINMSG) utils_GetSaveFileName},
	{"SelectDirectory", TYPE_MSGEX, (PLUGINMSG) utils_SelectDirectory},
	{"RegRead", TYPE_MSGEX, (PLUGINMSG) utils_regread},
	{"urlescape", TYPE_MSGEX, (PLUGINMSG) utils_urlescape},
	{"slash2backslash", TYPE_MSGEX, (PLUGINMSG) utils_slash2backslash},
	{"HostAllow", TYPE_MSG, (PLUGINMSG) utils_HostAllow},
	{"HostDeny", TYPE_MSG, (PLUGINMSG) utils_HostDeny},
	{"HostRemove", TYPE_MSG, (PLUGINMSG) utils_HostRemove},
	{"Listbox", TYPE_MSGEX, (PLUGINMSG) utils_Listbox},
	{"PopupMenu", TYPE_MSGEX, (PLUGINMSG) utils_PopupMenu},
	{"sendkey", TYPE_MSG, (PLUGINMSG) utils_sendkey},
#if (KMEL_PLUGIN_VER) > 0x0201 /* >= 1.0 */
	{"XPathGet", TYPE_MSGEX, (PLUGINMSG) utils_xpath_get},
	{"XPathSet", TYPE_MSG, (PLUGINMSG) utils_xpath_set},
	{"XPathSetDelimiter", TYPE_MSG, (PLUGINMSG) utils_xpath_set_delimiter},
#endif
	{NULL, (API_TYPE) NULL, NULL},
};

long DoMessage(const char *to, const char *from, const char *subject,
               long data1, long data2)
{
	if (to[0] == '*' || _stricmp(to, kPlugin.dllname) == 0) {
		PLUGINAPI *api;
		for (api = funcs; api->signature != NULL; api ++) {
			if (_stricmp(subject, api->signature) == 0) {
				switch (api->type) {
				case TYPE_MSG:
					api->pluginmsg((char *) data1, (char *) data2);
					break;
				case TYPE_MSGEX:
					long *p = (long *) data2;
					*p = api->pluginmsgex((char *) data1);
					break;
				}
				return 1;
			}
		}
	}
	return 0;
}

extern "C" {
	KMELEON_PLUGIN kmeleonPlugin *GetKmeleonPlugin() {
		return &kPlugin;
	}

	KMELEON_PLUGIN int DrawBitmap(DRAWITEMSTRUCT *dis) {
		return 14; // 14 = icon width
	}
}
